@@ -74,8 +74,11 @@ struct imp_sth_st {
 #define dbd_bind_ph             sqlite_bind_ph
 #define dbd_st_bind_col			sqlite_bind_col
 
-void sqlite_db_create_function(SV *dbh, const char *name, int argc, SV *func);
-void sqlite_db_create_aggregate( SV *dbh, const char *name, int argc, SV *aggr );
+void sqlite3_db_create_function(SV *dbh, const char *name, int argc, SV *func);
+void sqlite3_db_create_aggregate( SV *dbh, const char *name, int argc, SV *aggr );
+void sqlite_st_reset( SV *sth );
+int sqlite_bind_col( SV *sth, imp_sth_t *imp_sth, SV *col, SV *ref, IV sql_type, SV *attribs );
+int dbd_set_sqlite3_busy_timeout ( SV *dbh, int timeout );
 
 #ifdef SvUTF8_on
 
