@@ -2,15 +2,17 @@ package DBD::SQLite;
 
 use 5.005;
 use strict;
-use DBI;
+use DBI       ();
 use DynaLoader();
 
-use vars qw($err $errstr $state $drh $VERSION @ISA);
+use vars qw($VERSION @ISA);
 BEGIN {
-	$VERSION = '1.19_01';
+	$VERSION = '1.19_02';
 	@ISA     = ('DynaLoader');
 }
 
+use vars qw{$err $errstr $state $drh };
+
 __PACKAGE__->bootstrap($VERSION);
 
 $drh = undef;
@@ -595,6 +597,39 @@ Bugs should be reported via the CPAN bug tracker at
 
 L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=DBD-SQLite>
 
+=head1 TO DO
+
+There're several pended RT bugs/patches at the moment
+(mainly due to the lack of tests/patches or segfaults on tests).
+
+Here's the list.
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=41631>
+(patch required)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=40594>
+(patch required, and the following tests may break)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=30167>
+(need to see what is the best solution right now)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=34828>
+(segfault while testing under Windows)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=36836>
+(patch required)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=13631>
+(test required)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=35449>
+(break tests)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=29629>
+(patch required)
+
+Switch tests to Test::More to support more advanced testing behaviours
+
 =head1 AUTHOR
 
 Matt Sergeant E<lt>matt@sergeant.orgE<gt>
